module mod_globvar

	implicit none

	!------------------------------------------------
	!Parameters/Calibration - Quarterly Model
	!------------------------------------------------

	!Cash at hand
	integer :: ny_neg				!Negative cash at hand values
	integer :: ny_pos				!Positive cash at hand values
	integer :: ny					!Total number of cash at hand values
	real(8) :: min_y				!Minimum
	real(8) :: max_y				!Maximum
	real(8) :: curv_yneg			!Curvature of grid for negative points
	real(8) :: curv_ypos			!Curvature of grid for positive points
	real(8) :: dy_neg				!Increments for negative points
	real(8) :: dy_pos				!Increments for positive points
	
	!Cash at hand (extra fine grid for computing model moments)
	integer :: nyfine_neg
	integer :: nyfine_pos
	integer :: nyfine
	real(8) :: min_yfine
	real(8) :: max_yfine
	real(8) :: curv_yfineneg
	real(8) :: curv_yfinepos
	real(8) :: dyfine_neg
	real(8) :: dyfine_pos
	
	!Bonds
	integer :: nb
	real(8) :: max_b
	real(8) :: curv_b
	real(8) :: db
	
	!Housing and apartment parameters
	integer :: nh					!Number of house sizes
	real(8) :: min_h				!Minimum house size
	real(8) :: max_h				!Maximum house size
	real(8) :: curv_h				!Curvature of housing grid
	real(8) :: dh					!House size increments with curved grid
	real(8) :: hincrement			!House size increments with constant proportion h/h_1
	integer :: nu					!Housing match-specific shocks (1 or 2)
	integer :: ng					!Total number of housing states
	real(8) :: useparate			!Utility of housing when "separated"
	real(8) :: pseparate			!Probability of housing separation
	real(8) :: utilown				!Utility benefit of homeownership
	real(8) :: max_a				!Maximum apartment size
	real(8) :: rentratio			!Rent to price ratio
	real(8) :: pa					!Price of apartment space (5% steady state rent-price ratio)
	
	!Housing search
	integer :: nx					!Number of normalized housing submarkets
	real(8) :: curv_x
	real(8) :: phbase				!Baseline shadow price of housing
	real(8) :: min_xs				!Minimum selling price relative to ph (maximum seller discount)
	real(8) :: max_xs				!Maximum selling price relative to ph (minimum seller discount)
	real(8) :: min_xb				!Minimum buying price relative to ph (minimum buying premium)
	real(8) :: max_xb				!Maximum buying price relative to ph (maximum buying premium)
	real(8) :: dx
	real(8) :: curv_ms				!Curvature of seller's market matching function (< 1/2 for concavity)
	real(8) :: curv_mb				!Curvature of buyer's market matching function (< 1/2 for concavity)
	real(8) :: searchcost			!Price posting utility cost in selling market
	
	!Mortgage parameters
	integer :: nm
	real(8) :: max_ltv				!Maximum leverage at origination
	real(8) :: max_m				!Maximum pseudo-leverage -- make sure max_m >= max_ltv*max_ph
	real(8) :: curv_m
	real(8) :: dm
	
	!Foreclosure and REO parameters
	real(8) :: pflag				!Persistence of foreclosure flag
	real(8) :: fincloss				!Proportional income loss
	real(8) :: fdeficiency			!Deficiency judgment (fraction of liquid assets)
	real(8) :: fdiscount			!REO sale discount
	real(8) :: psearch				!Relative search efficiency of REO's

	!Preference parameters (CRRA specification)
	real(8) :: beta					!Discount factor
	real(8) :: gamma				!Coefficient of relative risk aversion
	real(8) :: eis					!Intertemporal elasticity of substitution (Epstein-Zin)
	real(8) :: theta				!Preference parameter (Epstein-Zin)
	real(8) :: ies					!Intratemporal elasticity of substitution (CES)
	real(8) :: cshare0				!Utility weight on non-housing consumption
	real(8) :: min_spend

	!Idiosyncratic labor productivity parameters (see Storesletten et al (2004))
	integer :: ns					!Number of permanent shocks (AR(1) process)
	real(8) :: mean_s				!Mean of permanent shock
	real(8) :: rho_s				!Persistence of permanent shock
	real(8) :: sig_s_e				!Standard deviation of permanent shock during expansions
	real(8) :: sig_s_r				!Standard deviation of permanent shock during recessions
	real(8) :: sig_s_avg			!Frequency-weighted average standard deviation of permanent shock
	real(8) :: cover_s				!Tauchen coverage of permanent shocks
	real(8) :: s_rich				!Average labor efficiency for ns relative to the average of labor effiency for all other s
	real(8) :: prob_getrich			!Probability of transitioning from ns - 1 to ns
	real(8) :: prob_stayrich		!Probability of staying in state ns
	integer :: ne					!Number of transitory shocks
	real(8) :: sig_e				!Standard deviation of transitory shock
	real(8) :: cover_e				!Tauchen coverage of transitory shocks

	!Depreciation
	real(8) :: deprec_k				!Quarterly depreciation rate of capital
	real(8) :: deprec_h				!Quarterly depreciation rate of housing

	!Production function parameters
	real(8) :: kshare_c				!Capital's share in the production of the consumption good (CCNR 2007)
	real(8) :: kshare_h				!Structure's share in the production of housing relative to labor (Van Nieuwerburgh et al 2011)
	real(8) :: lshare				!Land's share in the production of housing/apartments (Davis and Palumbo (2008))
	
	!Mortgage costs
	real(8) :: rscost				!Mortgage servicing cost
	real(8) :: rocost				!Mortgage origination cost
	
	!Government policy
	real(8) :: sellcost             !Transaction cost of selling (selling tax)
	
	!------------------------------------------------
	!Aggregate states
	!------------------------------------------------

	!Total factor productivity of composite good production
	integer :: nzc					!Number of shocks
	real(8) :: zc_steady			!Steady state composite good TFP
	real(8) :: sig_zc				!Standard deviation of TFP shock
	real(8) :: rho_zc				!Persistence
	real(8) :: cover_zc				!Tauchen coverage
	
	!Total factor productivity of housing construction
	integer :: nzh					!Number of shocks
	real(8) :: zh_steady			!Steady state housing TFP
	real(8) :: rho_zh				!Persistence
	real(8) :: sig_zh				!Variance
	real(8) :: cover_zh				!Tauchen coverage
	
	!Aggregate preference shocks to utility share of housing
	integer :: nhu
	real(8) :: sig_hu
	real(8) :: rho_hu
	real(8) :: cover_hu
	
	!Total number of aggregate shocks
	integer :: nz

	!Shadow price of housing
	integer :: nph
	real(8) :: min_ph
	real(8) :: max_ph
	real(8) :: curv_ph
	real(8) :: dph
	
	!Aggregate capital
	integer :: nk
	real(8) :: min_k
	real(8) :: max_k
	real(8) :: curv_k
	real(8) :: dk
	
	!Number of interpolation points for endogenous aggregate variables -- 2^(number of endogenous aggregate variables)
	integer :: ninterp
	
	!Total number of endogenous aggregate grid points
	integer :: nagg

	!------------------------------------------------
	!Prices and trade probabilities
	!------------------------------------------------
	real(8), dimension(:), allocatable :: prob_xs, prob_xb					!Submarket trade probabilities
	real(8), dimension(:,:,:), allocatable :: xbuy, xsell					!Submarket revenue
	real(8), dimension(:,:), allocatable :: labor							!Earnings
	real(8), dimension(:), allocatable :: ph								!Shadow price of housing
	real(8), dimension(:), allocatable :: kagg								!Aggregate capital
	real(8), dimension(:,:,:), allocatable :: rkprime						!Return on capital
	real(8), dimension(:,:,:), allocatable :: wprime						!Wage
	real(8), dimension(:,:), allocatable :: qb								!Bond price
	real(8), dimension(:,:), allocatable :: rb								!Risk-free rate
	real(8), dimension(:,:), allocatable :: qm								!Inverse of mortgage interest rate (qm = 1/(1 + rm))
	real(8), dimension(:,:,:,:,:,:), allocatable :: qm0						!Mortgage prices (includes closing costs, points, etc.)
	real(8), dimension(:), allocatable :: zc								!TFP of composite good production
	real(8), dimension(:), allocatable :: zh								!TFP of housing production
	real(8), dimension(:), allocatable :: cshare							!Preference shocks to utility share of housing

	!------------------------------------------------
	!Grids
	!------------------------------------------------
	real(8), dimension(:), allocatable :: mm								!Leverage
	real(8), dimension(:), allocatable :: yy								!Cash at hand
	real(8), dimension(:), allocatable :: yyfine							!Cash at hand (extra fine grid for computing aggregate statistics)
	real(8), dimension(:), allocatable :: bb								!Bonds
	real(8), dimension(:), allocatable :: hh								!Housing
	real(8), dimension(:), allocatable :: uu								!Housing utility
	real(8), dimension(:), allocatable :: xx								!Submarkets
	real(8), dimension(:), allocatable :: ss								!Permanent labor productivity shocks
	real(8), dimension(:), allocatable :: ee								!Transitory labor productivity shocks
	real(8), dimension(:), allocatable :: zzc								!TFP of composite good production
	real(8), dimension(:), allocatable :: zzh								!TFP of housing production
	real(8), dimension(:), allocatable :: hhu								!Preference shocks to utility share of housing
	real(8), dimension(:), allocatable :: pph								!Shadow price of housing
	real(8), dimension(:), allocatable :: kk								!Aggregate capital

	!------------------------------------------------
	!Transition probabilities and state variable correspondences
	!------------------------------------------------
	integer, dimension(:), allocatable :: zcfun, zhfun, hufun			!z --> (zc,zh,rf,hu) correspondence
	integer, dimension(:,:,:), allocatable :: zfun						!(zc,zh,hu) --> z correspondence
	integer, dimension(:), allocatable :: phfun, kfun					!agg --> (ph,kagg) correspondence
	integer, dimension(:,:), allocatable :: aggfun						!(ph,kagg) --> agg correspondence
	integer, dimension(:), allocatable :: hfun, ufun					!g --> (h,u) correspondence
	integer, dimension(:,:), allocatable :: gfun						!(h,u) --> g correspondence
	real(8), dimension(:,:,:,:), allocatable :: prob_ss					!Prob(s,s'|z,z'), ie prob(s'|s,z,z')
	real(8), dimension(:,:), allocatable :: prob_zz						!Prob(z,z'), ie prob(z'|z)
	real(8), dimension(:), allocatable :: stationary_z					!Stationary distribution of aggregate shocks
	real(8), dimension(:), allocatable :: emeas							!Pdf of transitory labor shock
	real(8) :: eavg, enorm												!Average labor productivity and normalization constant
	
	!------------------------------------------------
	!Forecasting and Simulation
	!------------------------------------------------
	real(8), dimension(:,:,:,:), allocatable :: forecast_coeff			!Coefficients for house price forecasting equation
	real(8), dimension(:,:), allocatable :: forecast_vec				!Aggregate state vector
	real(8), dimension(:,:,:), allocatable :: phprime					!Forecasting equation for the shadow price of housing
	real(8), dimension(:,:,:), allocatable :: kaggprime					!Forecasting equation for aggregate capital
	real(8), dimension(:,:,:), allocatable :: taxprime					!Tax rate for distributing intermediary profits/losses
	real(8), dimension(:,:,:,:), allocatable :: qmprime					!Forecasting equation for post-origination mortgage interest rates
	integer, dimension(:,:,:,:), allocatable :: iaggprime				!Interpolation vertices for aggregate state tomorrow given forecasts
	real(8), dimension(:,:,:,:), allocatable :: waggprime				!Interpolation weights for aggregate state tomorrow given forecasts
	real(8), dimension(:,:), allocatable :: timeseries					!Time series matrix

	!------------------------------------------------
	!Endogenous objects for optimization
	!------------------------------------------------
	!Banks:
	real(8), dimension(:,:,:), allocatable :: Jreo, psell_reo, xs_reo
	
	!Households -- Subperiod 3:
    real(8), dimension(:,:,:,:,:,:), allocatable :: Vown_f0, m_f0, bown_f0, cown_f0, hcown_f0, spendown_f0, bown_f0_sp1
    real(8), dimension(:,:,:,:,:), allocatable :: Vown_f1, bown_f1, cown_f1, hcown_f1, spendown_f1
    real(8), dimension(:,:,:,:), allocatable :: Vrent_f0, brent_f0, crent_f0, hcrent_f0, spendrent_f0, Vrent_f1, brent_f1, crent_f1, hcrent_f1, spendrent_f1
    integer, dimension(:,:,:,:,:,:), allocatable :: locm_f0, locbown_f0, locbown_f0_sp1
    integer, dimension(:,:,:,:,:), allocatable :: locbown_f1
    integer, dimension(:,:,:,:), allocatable :: locbrent_f0, locbrent_f1
    
    !Households -- Subperiods 1/2:
    real(8), dimension(:,:,:,:,:,:), allocatable :: Wown_f0, d_f0, xs_f0, xsratio_f0, psell_f0
    real(8), dimension(:,:,:,:,:), allocatable :: Wown_f1, xs_f1, xsratio_f1, psell_f1
    real(8), dimension(:,:,:,:), allocatable :: Wrent_f0, xb_f0, xbratio_f0, h_f0, pbuy_f0, Wrent_f1, xb_f1, xbratio_f1, h_f1, pbuy_f1
    integer, dimension(:,:,:,:,:,:), allocatable :: locxs_f0
    integer, dimension(:,:,:,:,:), allocatable :: locxs_f1
    integer, dimension(:,:,:,:), allocatable :: locxb_f0, loch_f0, locxb_f1, loch_f1
	
	!Welfare analysis of recourse policy
	real(8), dimension(:,:,:,:,:,:), allocatable :: Wown_f0_new
	real(8), dimension(:,:,:,:,:), allocatable :: Wown_f1_new
	real(8), dimension(:,:,:,:), allocatable :: Wrent_f0_new, Wrent_f1_new
	
	!------------------------------------------------
	!Distributions and REO housing stock (format: subperiod, owner/renter, good credit/bad credit)
	!------------------------------------------------
	real(8), dimension(:,:,:,:), allocatable :: mu1own_f0, mu2own_f0, mu3own_f0
	real(8), dimension(:,:,:), allocatable :: mu1own_f1, mu2own_f1, mu3own_f1
	real(8), dimension(:,:), allocatable :: mu1rent_f0, mu1rent_f1, mu2rent_f0, mu2rent_f1, mu3rent_f0, mu3rent_f1
	real(8), dimension(:), allocatable :: htot_reo
	
	!------------------------------------------------
	!Other
	!------------------------------------------------
	real(8), dimension(:,:,:,:), allocatable :: ylb    		            !Lower bound of y at mortgage origination
	integer :: load_data, load_series, load_params, valfn_type, matchfunction, monthly_quarterly, print_input, print_output, num_threads, num_threads_simul, rich_shock

	!Variables used to speed up and time value function iteration
	integer :: itermax, itermax_accel									!Maximum number of value function iterations
	real(8) :: toler													!Tolerance for convergence of value function iteration

end module mod_globvar